package com.simuwang.manage.service.competition;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.simuwang.base.common.conts.DateConst;
import com.simuwang.base.mapper.RcCompetitionResultMapper;
import com.simuwang.base.pojo.dos.RcCompetitionResultDO;
import com.simuwang.base.pojo.dto.competition.FzzqCompetitionResultDTO;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class FzzqCompetitionResultServiceService extends AbstractCompetitionResultService {

    private final RcCompetitionResultMapper rcCompetitionResultMapper;
    private final FundStrategyService fundStrategyService;

    public FzzqCompetitionResultServiceService(RcCompetitionResultMapper rcCompetitionResultMapper, FundStrategyService fundStrategyService) {
        this.rcCompetitionResultMapper = rcCompetitionResultMapper;

        this.fundStrategyService = fundStrategyService;
    }

    @Override
    public boolean isSupport(Integer competitionId) {
        return competitionId != null && competitionId == 48;
    }

    @Override
    protected List<FzzqCompetitionResultDTO> getCompetitionResult(Integer competitionId, String period) {
        List<FzzqCompetitionResultDTO> competitionResultDTOList = CollUtil.newArrayList();
        List<RcCompetitionResultDO> resultDOList = rcCompetitionResultMapper.queryCompetitionResult(competitionId, period);
        if (CollUtil.isEmpty(resultDOList)) {
            return competitionResultDTOList;
        }
        // 策略名称
        Map<Integer, String> strategyNameMap = fundStrategyService.getStrategyName(competitionId);
        competitionResultDTOList = resultDOList.stream().map(e -> convertToResultDTO(e, strategyNameMap.get(e.getStrategy()))).toList();

        // 策略分组和排名降序输出
        return competitionResultDTOList.stream()
                .sorted(Comparator.comparing(FzzqCompetitionResultDTO::getStrategyName).reversed()
                        .thenComparing(FzzqCompetitionResultDTO::getRank))
                .collect(Collectors.toList());
    }

    private FzzqCompetitionResultDTO convertToResultDTO(RcCompetitionResultDO resultDO, String strategyName) {
        FzzqCompetitionResultDTO resultDTO = BeanUtil.copyProperties(resultDO, FzzqCompetitionResultDTO.class);
        resultDTO.setStrategyName(strategyName);
        resultDTO.setRegisterNumber(resultDO.getFundRegisterNumber());
        resultDTO.setStartPriceDate(DateUtil.format(resultDO.getPrePriceDate(), DateConst.YYYY_MM_DD));
        resultDTO.setStartCumulativeNavWithdrawal(resultDO.getPreCumulativeNavWithdrawal() != null ? String.valueOf(resultDO.getPreCumulativeNavWithdrawal()) : null);
        resultDTO.setEndPriceDate(DateUtil.format(resultDO.getPriceDate(), DateConst.YYYY_MM_DD));
        resultDTO.setEndCumulativeNavWithdrawal(resultDO.getCumulativeNavWithdrawal() != null ? String.valueOf(resultDO.getCumulativeNavWithdrawal()) : null);
        resultDTO.setMaxdown(resultDO.getMaxdrawdown());
        resultDTO.setSharpeRatio(resultDO.getSharperatio());
        resultDTO.setCalmarRatio(resultDO.getCalmarratio());
        resultDTO.setSortinoRatio(resultDO.getSortinoratio());
        resultDTO.setProductScale(resultDO.getProductScale());
        if (resultDO.getRank() == null) {
            resultDTO.setRank(999999);
        }
        return resultDTO;
    }

}
